from personagem import Personagem
import direct.directbase.DirectStart
from direct.showbase.DirectObject import DirectObject
from pandac.PandaModules import *
from jogada import Jogada
from analisadorParedes import AnalisadorParedes
import random

class Fantasma(Personagem):

	listaDeSensores = dict()

	@staticmethod
	def preencheProbabilidades():
		for nome, sensor in Fantasma.listaDeSensores.iteritems():
			sensor.atribuiProbabilidades()

	def __init__(self, nome,x,y):
		Personagem.__init__(self)
		self.nome = nome
		self.ultimoMov = 'd'
		self.node = loader.loadModel("eggs/fantasma.egg")
		self.node.setScale(0.24,0.24,0.24)
		self.node.setX(x)
		self.node.setY(y)
		self.criaEsferaColisaoDoPusher()
		self.criaEsferaDeColisaoComPacDots()
		self.raioColFrontal = None
		self.listaDeJogadas = []
		self.estaSobrePacDot = False
		self.raioColLateral = []
		for i in range (0,4):
			self.raioColLateral.insert(i, None)
		self.analisadorParedes = AnalisadorParedes(self)
		self.estaPertoDeUmaParede = False

	def criaEsferaDeColisaoComPacDots(self):
		cs = CollisionSphere(0, 0, 1.7, 2)
		nomeAux = str(self.nome+"b")
		colNode = CollisionNode(nomeAux)
		colNode.setFromCollideMask(BitMask32(0x8))
		colNode.setIntoCollideMask(BitMask32(0x8))
		self.esferaColPacDot = self.node.attachNewNode(colNode)
		self.esferaColPacDot.node().addSolid(cs)

	def criaEsferaColisaoDoPusher(self):
		cs = CollisionSphere(0, 0, 2.3, 1.1)
		colNode = CollisionNode(self.nome)
		colNode.setFromCollideMask(BitMask32(0x2))
		colNode.setIntoCollideMask(BitMask32(0x2))
		self.esferaCol = self.node.attachNewNode(colNode)
		self.esferaCol.node().addSolid(cs)


	def mexeFantasma(self):

		numEscolhido = random.randrange(1,5)	
		if numEscolhido == 1:
			self.moveParaCima()		
		elif numEscolhido == 2:
			self.moveParaBaixo()
		elif numEscolhido == 3:
			self.moveParaLadoEsquerdo()
		else:
			self.moveParaLadoDireito()
	
	def adicionaJogada(self,nome,tempo,sensor):
		jogada = Jogada(nome,tempo,sensor)
		self.listaDeJogadas.append(jogada)

	def decideMovimentoQuandoEstaSobrePacDot(self,nomeDoSensor,tempo,objetoMaisPerto):

		sensor = Fantasma.listaDeSensores[nomeDoSensor]
		nomeDaAcao = sensor.sorteiaAcao()

		if nomeDaAcao == 'continuarEmCima':
			pass
		else:
			if objetoMaisPerto == None:
				self.mexeFantasma()
			else:
				nomeDoObjetoMaisPerto = objetoMaisPerto.getName()
				self.estaSobrePacDot = False
				if nomeDoObjetoMaisPerto == 'parede':
					self.andaNaPerpendicular()
					self.analisadorParedes.zeraDistancias()
				else:
					pass

		self.adicionaJogada(nomeDaAcao,tempo,nomeDoSensor)

	def andaNaPerpendicular(self):
		Personagem.andaPerpendicular(self)

	def andaPerpendicular(self,lado):
		if self.ultimoMov == 'c':
			if lado == 'esq':
				self.moveParaLadoEsquerdo()
			else:	
				self.moveParaLadoDireito()
		elif self.ultimoMov == 'b':
			if lado == 'esq':
				self.moveParaLadoDireito()
			else:
				self.moveParaLadoEsquerdo()
		elif self.ultimoMov == 'e':
			if lado == 'esq':
				self.moveParaBaixo()
			else:
				self.moveParaCima()
		else:
			if lado == 'esq':
				self.moveParaCima()
			else:
				self.moveParaBaixo()

	def moveFantasma(self, maisPerto, tempo, paredes):

		comQuemColidiu = maisPerto.getIntoNode()

		if comQuemColidiu.getName() == 'parede':
			self.estaPertoDeUmaParede = self.analisadorParedes.pertoDaParede(maisPerto)

		if self.estaSobrePacDot:
			self.decideMovimentoQuandoEstaSobrePacDot('sobrePacDot',tempo,comQuemColidiu)

		self.analisadorParedes.existeEntrada(paredes)

		if self.estaPertoDeUmaParede:
			if self.analisadorParedes.podeVirar[0]:
				self.andaPerpendicular('dir')
				self.analisadorParedes.zeraDistancias()

			elif self.analisadorParedes.podeVirar[1]:
				self.andaPerpendicular('esq')
				self.analisadorParedes.zeraDistancias()

			else:
				self.andaDirecaoContraria()
				self.analisadorParedes.inverteDistancias()

		elif self.analisadorParedes.podeVirar[0] or self.analisadorParedes.podeVirar[1]:
			
			numEscolhido = random.randrange(1,4)
			if numEscolhido <= 1:
				if self.analisadorParedes.podeVirar[0]:
					self.andaPerpendicular('dir')
				else:
					self.andaPerpendicular('esq')
				self.analisadorParedes.zeraDistancias()
			else:
				self.realizaAcao(comQuemColidiu, tempo, paredes)
			
		else:
			self.realizaAcao(comQuemColidiu, tempo, paredes)
		
		self.estaPertoDeUmaParede = False
		
	def realizaAcao(self,comQuemColidiu, tempo, paredes):

		nomeDoSensor = comQuemColidiu.getName()
		sensor = Fantasma.listaDeSensores[nomeDoSensor]
		
		nomeDaAcao = sensor.sorteiaAcao()

		if nomeDoSensor == 'parede': 
			if nomeDaAcao == 'andarNaDirecaoContraria' :
				self.andaDirecaoContraria()
				self.analisadorParedes.inverteDistancias()
			elif nomeDaAcao == 'avancarParaParede': 
				self.andaNaMesmaDirecao()

		elif nomeDoSensor == 'pacDot':
			if nomeDaAcao == 'avancarParaPacDot':
				self.andaNaMesmaDirecao()
			elif nomeDaAcao == 'andarNaDirecaoContraria':
				self.andaDirecaoContraria()
				self.analisadorParedes.inverteDistancias()
			elif nomeDaAcao == 'ficarParado': 
				pass
	
		elif nomeDoSensor == 'pacman':
			if nomeDaAcao == 'avancarParaPacman':
				self.andaNaMesmaDirecao()
			elif nomeDaAcao == 'andarNaDirecaoContraria':
				self.andaDirecaoContraria()
				self.analisadorParedes.inverteDistancias()
			elif nomeDaAcao == 'ficarParado': 
				pass

		elif nomeDoSensor.startswith('fantasma'):
			if nomeDaAcao == 'avancarParaPacDot':
				self.andaNaMesmaDirecao()
			elif nomeDaAcao == 'andarNaDirecaoContraria':
				self.andaDirecaoContraria()
				self.analisadorParedes.inverteDistancias()
			elif nomeDaAcao == 'ficarParado': 
				pass
		self.adicionaJogada(nomeDaAcao,tempo,nomeDoSensor)

	def lancaRaioVisao(self):
		raio = CollisionSegment(0,0,3,60,0,3)		
		self.criaRaioVisao(raio)

	def lancaRaiosLaterais(self):
		j = 0
		for i in range(-1,2):
			if not i == 0:
				raio = CollisionSegment(-2.4,0,3,-2.4,100*i,3)		
				self.criaRaioVisaoLateral(raio,j)
				j = j+1
		for i in range(-1,2):
			if not i == 0:
				raio = CollisionSegment(2.4,0,3,2.4,100*i,3)		
				self.criaRaioVisaoLateral(raio,j)
				j = j+1

	def criaRaioVisaoLateral(self,raio,i):
		colNode = CollisionNode('raioFantasmaLateral')		
		colNode.setFromCollideMask(BitMask32.bit(5))
		colNode.setIntoCollideMask(BitMask32.allOff())
		
		self.raioColLateral[i] = self.node.attachNewNode(colNode)
		self.raioColLateral[i].node().addSolid(raio)

	def criaRaioVisao(self,raio):
		colNode = CollisionNode('raioFantasma')		
		colNode.setFromCollideMask(BitMask32(0x5))
		colNode.setIntoCollideMask(BitMask32.allOff())
		self.raioColFrontal = self.node.attachNewNode(colNode)
		self.raioColFrontal.node().addSolid(raio)

	def cancelaRaiosLaterais(self):

		for i in range(0,4):
			if self.raioColLateral[i] != None:
				self.raioColLateral[i].removeNode()
				self.raioColLateral[i] = None

	def cancelaRaio(self):
		if self.raioColFrontal != None:
			self.raioColFrontal.removeNode()
			self.raioColFrontal = None
